/*
 * ioGame
 * Copyright (C) 2021 - 2023  渔民小镇 （262610965@qq.com、luoyizhu@gmail.com） . All Rights Reserved.
 * # iohao.com . 渔民小镇
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package com.iohao.mmo;

import com.iohao.game.action.skeleton.ext.spring.ActionFactoryBeanForSpring;
import com.iohao.game.bolt.broker.client.AbstractBrokerClientStartup;
import com.iohao.game.bolt.broker.server.BrokerServer;
import com.iohao.game.external.core.ExternalServer;
import com.iohao.game.external.core.config.ExternalGlobalConfig;
import com.iohao.game.external.core.netty.simple.NettyRunOne;
import com.iohao.mmo.bag.BagLogicServer;
import com.iohao.mmo.broker.MyBrokerServer;
import com.iohao.mmo.common.config.MyGlobalSetting;
import com.iohao.mmo.equip.EquipLogicServer;
import com.iohao.mmo.external.MyExternalServer;
import com.iohao.mmo.level.LevelLogicServer;
import com.iohao.mmo.login.LoginLogicServer;
import com.iohao.mmo.mail.MailLogicServer;
import com.iohao.mmo.map.MapLogicServer;
import com.iohao.mmo.person.PersonLogicServer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;

import java.util.List;

/**
 * 游戏服务器启动类
 *
 * @author 渔民小镇
 * @date 2023-07-21
 */
@Slf4j
@SpringBootApplication
public class OneApplication {
    public static void main(String[] args) {
        SpringApplication.run(OneApplication.class, args);

        extractedConfig();

        // 游戏逻辑服列表
        List<AbstractBrokerClientStartup> logicServers = listLogic();

        // 游戏对外服
        ExternalServer externalServer = new MyExternalServer()
                .createExternalServer(ExternalGlobalConfig.externalPort);

        BrokerServer brokerServer = new MyBrokerServer()
                .createBrokerServer();

        new NettyRunOne()
                // 游戏对外服
                .setExternalServer(externalServer)
                // Broker（游戏网关服）
                .setBrokerServer(brokerServer)
                // 游戏逻辑服列表
                .setLogicServerList(logicServers)
                // 启动游戏服务器
                .startup();
    }

    private static List<AbstractBrokerClientStartup> listLogic() {
        // 登录
        LoginLogicServer loginLogicServer = new LoginLogicServer();
        // 人物
        PersonLogicServer personLogicServer = new PersonLogicServer();
        // 地图
        MapLogicServer mapLogicServer = new MapLogicServer();
        // 等级
        LevelLogicServer levelLogicServer = new LevelLogicServer();
        // 背包
        BagLogicServer bagLogicServer = new BagLogicServer();
        // 邮件
        MailLogicServer mailLogicServer = new MailLogicServer();
        //装备
        EquipLogicServer equipLogicServer = new EquipLogicServer();

        // 游戏逻辑服列表
        return List.of(
                loginLogicServer
//                , personLogicServer
//                , mapLogicServer
//                , levelLogicServer
                , bagLogicServer
                , mailLogicServer
                , equipLogicServer
        );
    }

    private static void extractedConfig() {
        // 统一的默认配置
        MyGlobalSetting.defaultSetting();
    }

    @Bean
    public ActionFactoryBeanForSpring actionFactoryBean() {
        // 将业务框架交给 spring 管理
        return ActionFactoryBeanForSpring.me();
    }
}
